package Addressbook;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: Task.java,v 1.2 2002/03/11 22:32:35 kstaken Exp $
 */
 
import java.io.*;
import java.net.*;
import java.beans.*;
import javax.servlet.*;
import javax.servlet.jsp.*;
import javax.servlet.http.*;

/**
Servlet class for the addressbook

This class acts as a controller, forwarding HTML request onto the correct class/jsp page
depending on the value of request.action

*/
public class Task extends HttpServlet {

   public void doPost(HttpServletRequest request, HttpServletResponse response ) throws ServletException, IOException {

      JspFactory jspFactory = null;
      PageContext pageContext = null;
      HttpSession session = null;
      Addressbook.Group group = null;

      String action = null;
      
      try {
      
         // If the Group bean hasn't been added to the session alerady, add it
         // this allows the user to go directly to ANY page
         jspFactory = JspFactory.getDefaultFactory();
         pageContext = jspFactory.getPageContext(this, request, response, "/error.jsp", true, 8192, true );
         session = pageContext.getSession();
         
         group = (Group)pageContext.getAttribute("group",PageContext.SESSION_SCOPE);
         if (group == null) {
            group=(Addressbook.Group)Beans.instantiate(this.getClass().getClassLoader(), "Addressbook.Group" );
            pageContext.setAttribute("group", group, PageContext.SESSION_SCOPE );
         }

      
         // Since case matters, we need to check for both....
         if ( request.getParameter("action") != null ) {
            action = request.getParameter("action");
         }
         else {
            action = request.getParameter("ACTION");
         }
         
         // Decide where to send the user depending on value of action
         if ( action != null ) {
            // Add a Contact
            if ( action.equalsIgnoreCase("addform") ) {
               gotoPage("/addContact.jsp",request,response);
            } else if ( action.equalsIgnoreCase("addcontact") ) {
               AddContact contact = new AddContact();
               contact.add(request,response);
               
               ListContacts lcontact = new ListContacts();
               lcontact.list(request,response);
               gotoPage("/listContacts.jsp",request,response);
               
            // List all contacts
            } else if ( action.equalsIgnoreCase("listcontacts") ) {
               ListContacts contact = new ListContacts();
               contact.list(request,response);
               gotoPage("/listContacts.jsp",request,response);
            
            // Show individual Contact detail
            } else if ( action.equalsIgnoreCase("showcontact") ) {
               gotoPage("/showContact.jsp",request,response);

            // Show individual XML file
            } else if ( action.equalsIgnoreCase("showxml") ) {
               gotoPage("/showXML.jsp",request,response);
                              
            // Edit a contact
            } else if ( action.equalsIgnoreCase("editcontactform") ) {
               gotoPage("/editContactForm.jsp",request,response);
            } else if ( action.equalsIgnoreCase("editcontact") ) {
               EditContact contact = new EditContact();
               contact.edit(request,response);
               
               ListContacts lcontact = new ListContacts();
               lcontact.list(request,response);
               gotoPage("/listContacts.jsp",request,response);
            
            // Search for a contact
            } else if ( action.equalsIgnoreCase("searchform") ) {
               gotoPage("/searchContacts.jsp",request,response);
            } else if ( action.equalsIgnoreCase("attributesearch") ) {
               SearchContacts contact = new SearchContacts();
               contact.attributeSearch(request,response);
               gotoPage("/listContacts.jsp",request,response);
            } else if ( action.equalsIgnoreCase("xpathsearch") ) {
               SearchContacts contact = new SearchContacts();
               contact.xpathSearch(request,response);
               gotoPage("/listContacts.jsp",request,response);
            
            // Delete a contact
            } else if ( action.equalsIgnoreCase("deletecontact") ) {
               DeleteContact contact = new DeleteContact();
               contact.delete(request,response);
               
               ListContacts lcontact = new ListContacts();
               lcontact.list(request,response);
               gotoPage("/listContacts.jsp",request,response);
               
            } else gotoPage("/error.jsp",request,response);
         }
         else {
            gotoPage("/error.jsp",request,response);
         }
         
      } catch( Exception e ) {
         e.printStackTrace();

	  // there's not much else we can do if the response is committed
	  if (response.isCommitted())
	     return;

         // Catch the exception and send the user to the error page	 
         if (e.getMessage() != null ) {
            response.sendRedirect("/Addressbook/error.jsp?error=" + URLEncoder.encode(e.getMessage()) );
         }
         else {
            response.sendRedirect("/Addressbook/error.jsp" );
         }
      }
   }
   
   // Treat a form-get as a post, seding user to doPost
   public void doGet(HttpServletRequest request, HttpServletResponse response ) throws ServletException, IOException {
   
      doPost(request,response);
   
   }

   // Forward the user on to the pages specified in address
   private void gotoPage( String address, HttpServletRequest request, HttpServletResponse response ) throws ServletException, IOException {
   
      RequestDispatcher dispatcher = getServletContext().getRequestDispatcher(address);
      dispatcher.forward(request, response);
   
   }

}

