package org.apache.xindice.client.corba;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: EncodedBufferConverter.java,v 1.1.1.1 2001/12/06 19:33:53 bradford Exp $
 */

import org.apache.xindice.xml.*;
import org.apache.xindice.xml.dom.*;
import org.apache.xindice.client.corba.db.*;
import org.apache.xindice.core.data.*;
import org.apache.xindice.util.*;

import org.w3c.dom.*;

import java.io.*;

/**
 * Converts an EncodedBuffer into a Configuration object.
 */
public class EncodedBufferConverter {
   private static final byte[] EmptyBytes = new byte[0];
   
   /**
     * Converts an EncodedBuffer into a Configuration object. Assumes the
     * buffer contains valid XML data.
     *
     * @param buffer the buffer containing XML to convert
     * @return A Configuration object encapsualting the XML.
     */
   public static Configuration convertToConfiguration(EncodedBuffer buffer) {
      Configuration config = null;

      Document doc = getDocument(buffer);
      config = new Configuration(doc.getDocumentElement(), false);

      return config;
   }

   /**
     * Converts an EncodedBuffer into a Document object. Assumes the
     * buffer contains valid XML data.
     *
     * @param buffer the buffer containing XML to convert
     * @return A Document object encapsualting the XML.
     */
   public static Document convertToDocument(EncodedBuffer buffer) {
      return getDocument(buffer);
   }

   /**
     * Converts a Document object into an EncodedBuffer.
     *
     * @param doc the Node to convert to an EncodedBuffer
     * @param encoding the encoding to use when converting the doc
     * @return An EncodedBuffer containing the serialized document
     */
   public static EncodedBuffer convertFromDocument(Node node) {
      EncodedBuffer result = new EncodedBuffer();
      result.sym = EmptyBytes;
      result.buf = TextWriter.toString(node).getBytes();
      result.stamp = -1;
      return result;
   }

   /**
      * Parses the buffer into a Document.
      */
   private static Document getDocument(EncodedBuffer buffer) {
      String buf = new String(buffer.buf);

      try {
         // Parse the buffer
         return DOMParser.toDocument(buf);
      }
      catch ( Exception e ) {
         //org.apache.xindice.Debug.println("Error Converting Buffer '" + buf + "'");
         return null;
      }
   }

   /*
    * Releases all resources associated with this object
    */
   public void remove() {
   }
}




