package org.apache.xindice.core.xupdate;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: XUpdateImpl.java,v 1.2 2002/01/16 09:48:39 kstaken Exp $
 */

import org.apache.xindice.core.Collection;
import org.apache.xindice.core.data.*;
import org.apache.xindice.xml.dom.*;
import org.apache.xindice.xml.*;

import org.xmldb.xupdate.lexus.commands.*;
import org.xmldb.xupdate.lexus.*;
import org.infozone.tools.xml.queries.*;

import java.util.*;

import org.w3c.dom.*;
import org.xmldb.xupdate.lexus.*;
import org.xml.sax.*;

/**
 * Provides Collection and document based XUpdate capabilities.
 *
 * For more detail about XUpdate look at the
 * <a href="http://www.xmldb.org/xupdate/xupdate-wd.html">XUpdate Working Draft</a>.
 */
public class XUpdateImpl extends XUpdateQueryImpl {
   protected int nodesModified = 0;
   protected NamespaceMap nsMap;

    /**
     * If set to true, then namespaces set explicitly via an API call will take precendence.
     * If set to false, then namespaces set implicitly within query string will take precedence.
     */
    private static final boolean API_NS_PRECEDENCE = true;

    /**
     * Set the namespace map to be used when resolving queries
     */
    public void setNamespaceMap(NamespaceMap nsMap) {
       if (nsMap == null)
          return;

       if (this.nsMap == null) {
          this.nsMap = nsMap;
       }
       else {
          this.nsMap.includeNamespaces(nsMap, API_NS_PRECEDENCE);
       }
    }


    /**
     * Sets the query string to be used when executing update
     */
    public void setQString(String query) throws SAXException {
       super.setQString(query);
       if (nsMap == null) {
          nsMap = new NamespaceMap();
       }
       nsMap.includeNamespaces(_namespaces, !API_NS_PRECEDENCE);
    }


    /**
    * Execute the XUpdate commands against a document.
    */
   public void execute(Node contextNode) throws Exception {
      CommandObject currentCommand = new DefaultCommand(contextNode);
      Enumeration commands = _query[0].elements();
      Enumeration attributes = _query[1].elements();
      Enumeration characters = _query[2].elements();
      Node origNode = contextNode;
      CommandObject.getXPath().setNamespace(nsMap.getContextNode());

      while (commands.hasMoreElements()) {
         int id = ((Integer) commands.nextElement()).intValue();

         if ( id == _commands.ATTRIBUTES ) {
            currentCommand.submitAttributes((Hashtable) attributes.nextElement());
         }
         else if ( id == _commands.CHARACTERS ) {
            currentCommand.submitCharacters((String) characters.nextElement());
         }
         else if ( id > 0 ) {
            if ( ! currentCommand.submitInstruction(id) ) {
               _commands.setContextNode(contextNode);
               currentCommand = _commands.commandForID(id);
               if (currentCommand == null) {
                  throw new Exception("operation can not have any XUpdate-instruction !");
               }
               currentCommand.reset();
            }
         }
         else {
            if ( ! currentCommand.executeInstruction() ) {
               contextNode = currentCommand.execute();
               currentCommand = new DefaultCommand(contextNode);
            }
         }
      }

      if ( origNode instanceof CompressedNode ) {
         CompressedNode cn = (CompressedNode)origNode;
         if ( cn.isDirty() )
            nodesModified++;
      }
   }

   /**
    * Execute the set of XUpdate commands against a collection.
    *
    * @param contextNode Description of Parameter
    * @exception Exception Description of Exception
    */
   public void execute(Collection col) throws Exception {

      int attribIndex = 0;
      for (int i = 0; i < _query[0].size(); i++) {
         int cmdID = ((Integer) _query[0].elementAt(i)).intValue();

         if ( cmdID == _commands.ATTRIBUTES ) {
            Hashtable attribs = (Hashtable) _query[1].elementAt(attribIndex);
            attribIndex++;
            String selector = (String) attribs.get("select");

            // If we found an XPath selector we need to execute the commands.
            if (selector != null) {
               NodeSet ns = col.queryCollection("XPath", selector, nsMap);
               Document lastDoc = null;
               while (ns != null && ns.hasMoreNodes()) {
                  DBNode node = (DBNode)ns.getNextNode();
                  Document doc = node.getOwnerDocument();
                  
                  if ( doc == lastDoc )
                     continue; // We only have to process it once
                  else
                     lastDoc = doc;
                  
                  NodeSource source = node.getSource();

                  Node contextNode = doc.getDocumentElement();
                  execute(contextNode);

                  col.setDocument(source.getKey(), doc);
               }
            }
         }
      }
   }
   
   public int getModifiedCount() {
      return nodesModified;
   }
}

