package org.apache.xindice.server;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: Service.java,v 1.1.1.1 2001/12/06 19:33:55 bradford Exp $
 */

import org.apache.xindice.util.*;

/**
 * Services are implemented to provide a daemon process for various
 * purposes.  The most general implementation of a Service is to
 * provide a Socket based service, but Services can be written
 * to function in just about any capacity.
 *
 * @see org.apache.xindice.server.ServiceManager
 */

public interface Service extends Named {
   /* Active state */

   static final int STATE_STOPPED = 0;
   static final int STATE_STARTED = 1;
   static final int STATE_SUSPENDED = 2;

   /* Result values */

   static final int RESULT_OK = 0;
   static final int RESULT_BUSY = 1;
   static final int RESULT_FAILED = 2;
   static final int RESULT_INVALID = 3;

   /* Methods */

   /**
    * status returns the current operational status of this Service.  This
    * value can either be STATE_STOPPED, STATE_STARTED, or STATE_SUSPENDED.
    *
    * @return The Service's status
    */
   int status();

   /**
    * statusMessage returns a textual message about the Service's
    * operational state.
    *
    * @return The Service's status as a text message
    */
   String statusMessage();

   /**
    * initialize will initialize the Service based on its current
    * configuration.  RESULT_INVALID will be returned if the Service's
    * configuration is invalid.
    *
    * @return The initialization status
    */
   int initialize();

   /**
    * start will attempt to start the Service.  Depending on whether the
    * Service was happy about being started or not, a value of RESULT_OK
    * should be returned.
    *
    * @return The result of the start attempt
    */
   int start();

   /**
    * stop will attempt to stop the Service.  Depending on whether the
    * Service was happy about being stopped or not, a value of RESULT_OK
    * should be returned.
    *
    * @return The result of the stop attempt
    */
   int stop();

   /**
    * suspend suspends execution of the Service.  It basically puts it into
    * a paused state until it is resumed or started again.  Depending on
    * whether the Service was happy about being suspended or not, a value of
    * RESULT_OK should be returned.
    *
    * @return The result of the suspend attempt
    */
   int suspend();

   /**
    * resume resumes exection of a suspended Service.  It will also start a
    * stopped Service.  Depending on whether the Service was happy about
    * being resumed or not, a value of RESULT_OK should be returned.
    *
    * @return The result of the resume attempt
    */
   int resume();

   /**
    * uninitialize will uninitialize the Service based on its current
    * configuration and will clean up any existing resources for that
    * configuration.  RESULT_INVALID will be returned if the Service's
    * configuration is invalid.
    *
    * @return The uninitialization status
    */
   int uninitialize();
}
