package org.apache.xindice.xml.dom;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: DOMParser.java,v 1.4 2001/12/18 17:37:09 bradford Exp $
 */

import java.io.*;
import java.util.*;

import org.apache.xindice.core.data.*;
import org.apache.xindice.util.*;
import org.apache.xindice.xml.*;

import javax.xml.parsers.*;

import org.w3c.dom.*;
import org.xml.sax.*;
import org.xml.sax.ext.*;
import org.xml.sax.helpers.*;

/**
 * DOMParser is the Xindice compressed DOM parser.
 */

public final class DOMParser extends DefaultHandler implements DeclHandler, LexicalHandler {
   private SAXParser sp;
   private ErrorHandler errors = null;
   private EntityResolver entities = null;
   
   private Document doc;
   private Node context;
   private int state = -1;
   private StringBuffer buf;
   private ObjectStack states = new ObjectStack();
   
   public DOMParser() throws XindiceException {
      try {
         sp = getSAXParser();
      }
      catch ( Exception e ) {
         throw new XindiceException("Error creating parser");
      }
   }
   
   public void setErrorHandler(ErrorHandler errors) throws SAXException {
      this.errors = errors;
      if ( sp != null ) {
         XMLReader xr = sp.getXMLReader();
         xr.setErrorHandler(errors);
      }
   }
   
   public void setEntityResolver(EntityResolver entities) throws SAXException {
      this.entities = entities;
      if ( sp != null ) {
         XMLReader xr = sp.getXMLReader();
         xr.setEntityResolver(entities);
      }
   }

   public void parse(String xml) throws SAXException, IOException {
      parse(new StringReader(xml));
   }
   
   public void parse(Reader input) throws SAXException, IOException {
      parse(new InputSource(input));
   }
   
   public void parse(byte[] xml) throws SAXException, IOException {
      parse(new ByteArrayInputStream(xml));
   }
   
   public void parse(Value value) throws SAXException, IOException {
      parse(value.getData());
   }

   public void parse(InputSource source) throws SAXException, IOException {
      sp.parse(source, this);
   }
   
   public void parse(InputStream input) throws SAXException, IOException {
      sp.parse(input, this);
   }
   
   public static Document toDocument(Reader input) throws XindiceException {
      try {
         DOMParser dp = new DOMParser();
         dp.parse(input);
         return dp.getDocument();
      }
      catch ( Exception e ) {
         throw new XindiceException("Error parsing Document", e);
      }
   }
   
   public static Document toDocument(byte[] xml) throws XindiceException {
      try {
         DOMParser dp = new DOMParser();
         dp.parse(xml);
         return dp.getDocument();
      }
      catch ( Exception e ) {
         throw new XindiceException("Error parsing Document", e);
      }
   }
   
   public static Document toDocument(Value value) throws XindiceException {
      try {
         DOMParser dp = new DOMParser();
         dp.parse(value);
         return dp.getDocument();
      }
      catch ( Exception e ) {
         throw new XindiceException("Error parsing Document", e);
      }
   }

   public static Document toDocument(InputSource source) throws XindiceException {
      try {
         DOMParser dp = new DOMParser();
         dp.parse(source);
         return dp.getDocument();
      }
      catch ( Exception e ) {
         throw new XindiceException("Error parsing Document", e);
      }
   }
   
   public static Document toDocument(InputStream input) throws XindiceException {
      try {
         DOMParser dp = new DOMParser();
         dp.parse(input);
         return dp.getDocument();
      }
      catch ( Exception e ) {
         throw new XindiceException("Error parsing Document", e);
      }
   }
   
   public static Document toDocument(String xml) throws XindiceException {
      try {
         DOMParser dp = new DOMParser();
         dp.parse(xml);
         return dp.getDocument();
      }
      catch ( Exception e ) {
         throw new XindiceException("Error parsing Document", e);
      }
   }
   
   /**
    * Returns the parsed Document
    */
   public Document getDocument() {
      return doc;
   }
   
   private void setReaderProperty(XMLReader xr, String name, Object value) {
      try {
         xr.setProperty(name, value);
      }
      catch ( Exception e ) {
      }
   }

   private void setReaderFeature(XMLReader xr, String name, boolean value) {
      try {
         xr.setFeature(name, value);
      }
      catch ( Exception e ) {
      }
   }
   
   private SAXParser getSAXParser() throws Exception {
      SAXParserFactory f = SAXParserFactory.newInstance();
      SAXParser sp = f.newSAXParser();
      
      XMLReader xr = sp.getXMLReader();
      setReaderFeature(xr, "http://xml.org/sax/features/namespaces", true);
      setReaderFeature(xr, "http://xml.org/sax/features/validation", false);
      setReaderFeature(xr, "http://xml.org/sax/features/external-general-entities", false);
      setReaderFeature(xr, "http://xml.org/sax/features/external-parameter-entities", false);
      setReaderFeature(xr, "http://xml.org/sax/features/namespace-prefixes", true);
      setReaderProperty(xr, "http://xml.org/sax/properties/lexical-handler", this);
      setReaderProperty(xr, "http://xml.org/sax/properties/declaration-handler", this);
      
      if ( errors != null )
         xr.setErrorHandler(errors);
      if ( entities != null )
         xr.setEntityResolver(entities);
      
      return sp;
   }
   
   private void pushState(int newState) {
      Integer i = new Integer(state);
      states.push(i);
      state = newState;
   }
   
   private void popState() {
      Integer i = (Integer)states.pop();
      state = i != null ? i.intValue() : -1;
   }
   
   
   // Implementation of the EntityResolver interface
   
   public InputSource resolveEntity(String publicId, String systemId) throws SAXException {
      return null;
   }
   
   
   // Implementation of DTDHandler interface
   
   public void notationDecl (String name, String publicId, String systemId) throws SAXException {
   }
   
   public void unparsedEntityDecl (String name, String publicId, String systemId, String notationName) throws SAXException {
   }
   
   
   // Implementation of ContentHandler interface
   
   public void setDocumentLocator(Locator locator) {
   }
   
   public void startDocument() throws SAXException {
      doc = new DocumentImpl();
      context = doc;
      pushState(Node.DOCUMENT_NODE);
   }
   
   public void endDocument() throws SAXException {
      context = null;
      popState();
   }
   
   public void startPrefixMapping(String prefix, String uri) throws SAXException {
   }
   
   public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
      Element e;
      if ( uri != null && uri.length() > 0 )
         e = doc.createElementNS(uri, qName);
      else
         e = doc.createElement(localName);

      for ( int i = 0; i < attributes.getLength(); i++ ) {
         String attrURI = attributes.getURI(i);;
         String attrQName = attributes.getQName(i);
         String attrValue = attributes.getValue(i);

         boolean isNsDef = attrQName != null && attrQName.startsWith("xmlns")
                        && (attrQName.length() == 5 || attrQName.charAt(5) == ':');
         
         if ( isNsDef || ( attrURI != null && attrURI.length() > 0 ) )
            e.setAttributeNS(attrURI, attrQName, attrValue);
         else
            e.setAttribute(attributes.getLocalName(i), attrValue);
      }
      
      context.appendChild(e);
      pushState(Node.ELEMENT_NODE);
      context = e;
   }
   
   public void endElement(String uri, String localName, String qName) throws SAXException {
      context = context.getParentNode();
      popState();
   }
   
   public void characters(char ch[], int start, int length) throws SAXException {
      switch ( state ) {
         case Node.CDATA_SECTION_NODE:
            buf.append(ch, start, length);
            break;
            
         default:
            String s = new String(ch, start, length);
            context.appendChild(doc.createTextNode(s));
      }
   }
   
   public void ignorableWhitespace(char ch[], int start, int length) throws SAXException {
      switch ( state ) {
         case Node.CDATA_SECTION_NODE:
            buf.append(ch, start, length);
            break;
            
         default:
            String s = new String(ch, start, length);
            context.appendChild(doc.createTextNode(s));
      }
   }
   
   public void processingInstruction(String target, String data) throws SAXException {
      context.appendChild(doc.createProcessingInstruction(target, data));
   }
   
   public void skippedEntity(String name) throws SAXException {
   }
   

   // Implementation of the ErrorHandler interface

   public void warning(SAXParseException e) throws SAXException {
   }
   
   public void error(SAXParseException e) throws SAXException {
   }
   
   public void fatalError(SAXParseException e) throws SAXException {
      throw e;
   }
   
   
   // Implementation of the DeclHandler interface
   
   public void elementDecl(String name, String model) throws SAXException {
   }
   
   public void attributeDecl(String eName, String aName, String type, String valueDefault, String value) throws SAXException {
   }
   
   public void internalEntityDecl(String name, String value) throws SAXException {
   }
   
   public void externalEntityDecl(String name, String publicId, String systemId) throws SAXException {
   }

   
   // Implementation of the LexicalHandler interface
   
   public void startDTD(String name, String publicId, String systemId) throws SAXException {
   }

   public void endDTD() throws SAXException {
   }

   public void startEntity(String name) throws SAXException {
   }

   public void endEntity(String name) throws SAXException {
   }

   public void startCDATA() throws SAXException {
      pushState(Node.CDATA_SECTION_NODE);
      buf = new StringBuffer();
   }

   public void endCDATA () throws SAXException {
      context.appendChild(doc.createCDATASection(buf.toString()));
      buf = null;
      popState();
   }

   public void comment(char ch[], int start, int length) throws SAXException {
      String s = new String(ch, start, length);
      context.appendChild(doc.createComment(s));
   }
   
}
